@@ -9,7 +9,28 @@ import unittest
 from linkgrammar import Parser, Linkage, ParseOptions, Link
 import _clinkgrammar as clg
 
-class ParseOptionsTestCase(unittest.TestCase):
+import os
+
+def setUpModule():
+    datadir = os.getenv("LINK_GRAMMAR_DATA", "");
+    if datadir:
+        # Make sure we use an absolute path, else "Error opening word file..."
+        datadir = os.path.abspath(datadir)
+        clg.dictionary_set_data_dir(datadir)
+
+# The tests are run in alphabetical order....
+#
+# First test: test the test framework itself ...
+class AAALinkTestCase(unittest.TestCase):
+    def test_link_display_with_identical_link_type(self):
+        self.assertEqual(unicode(Link('Left','Link','Link','Right')),
+                         u'Left-Link-Right')
+
+    def test_link_display_with_identical_link_type(self):
+        self.assertEqual(unicode(Link('Left','Link','Link*','Right')),
+                         u'Left-Link-Link*-Right')
+
+class BParseOptionsTestCase(unittest.TestCase):
     def test_setting_verbosity(self):
         po = ParseOptions()
         po.verbosity = 2
@@ -74,19 +95,6 @@ class ParseOptionsTestCase(unittest.TestCase):
         po = ParseOptions()
         self.assertRaises(ValueError, setattr, po, "max_null_count", -1)
 
-    def test_setting_null_block(self):
-        po = ParseOptions()
-        po.null_block = 3
-        self.assertEqual(clg.parse_options_get_null_block(po._po), 3)
-
-    def test_setting_null_block_to_non_integer_raises_type_error(self):
-        po = ParseOptions()
-        self.assertRaises(TypeError, setattr, po, "null_block", "a")
-
-    def test_setting_null_block_to_negative_number_raises_value_error(self):
-        po = ParseOptions()
-        self.assertRaises(ValueError, setattr, po, "null_block", -1)
-
     def test_setting_short_length(self):
         po = ParseOptions()
         po.short_length = 3
@@ -100,19 +108,6 @@ class ParseOptionsTestCase(unittest.TestCase):
         po = ParseOptions()
         self.assertRaises(ValueError, setattr, po, "short_length", -1)
 
-    def test_setting_max_sentence_length(self):
-        po = ParseOptions()
-        po.max_sentence_length = 3
-        self.assertEqual(clg.parse_options_get_max_sentence_length(po._po), 3)
-
-    def test_setting_max_sentence_length_to_non_integer_raises_type_error(self):
-        po = ParseOptions()
-        self.assertRaises(TypeError, setattr, po, "max_sentence_length", "a")
-
-    def test_setting_max_sentence_length_to_negative_number_raises_value_error(self):
-        po = ParseOptions()
-        self.assertRaises(ValueError, setattr, po, "max_sentence_length", -1)
-
     def test_setting_islands_ok(self):
         po = ParseOptions()
         po.islands_ok = True
@@ -135,32 +130,6 @@ class ParseOptionsTestCase(unittest.TestCase):
         po = ParseOptions()
         self.assertRaises(TypeError, setattr, po, "max_parse_time", "a")
 
-    def test_setting_screen_width(self):
-        po = ParseOptions()
-        po.screen_width = 3
-        self.assertEqual(clg.parse_options_get_screen_width(po._po), 3)
-
-    def test_setting_screen_width_to_non_integer_raises_type_error(self):
-        po = ParseOptions()
-        self.assertRaises(TypeError, setattr, po, "screen_width", "a")
-
-    def test_setting_screen_width_to_negative_number_raises_value_error(self):
-        po = ParseOptions()
-        self.assertRaises(ValueError, setattr, po, "screen_width", -1)
-
-    def test_setting_allow_null(self):
-        po = ParseOptions()
-        po.allow_null = True
-        self.assertEqual(po.allow_null, True)
-        self.assertEqual(clg.parse_options_get_allow_null(po._po), 1)
-        po.allow_null = False
-        self.assertEqual(po.allow_null, False)
-        self.assertEqual(clg.parse_options_get_allow_null(po._po), 0)
-
-    def test_setting_allow_null_to_non_boolean_raises_type_error(self):
-        po = ParseOptions()
-        self.assertRaises(TypeError, setattr, po, "allow_null", "a")
-
     def test_setting_display_morphology(self):
         po = ParseOptions()
         po.display_morphology = True
@@ -170,19 +139,6 @@ class ParseOptionsTestCase(unittest.TestCase):
         self.assertEqual(po.display_morphology, False)
         self.assertEqual(clg.parse_options_get_display_morphology(po._po), 0)
 
-    def test_setting_display_walls(self):
-        po = ParseOptions()
-        po.display_walls = True
-        self.assertEqual(po.display_walls, True)
-        self.assertEqual(clg.parse_options_get_display_walls(po._po), 1)
-        po.display_walls = False
-        self.assertEqual(po.display_walls, False)
-        self.assertEqual(clg.parse_options_get_display_walls(po._po), 0)
-
-    def test_setting_display_walls_to_non_boolean_raises_type_error(self):
-        po = ParseOptions()
-        self.assertRaises(TypeError, setattr, po, "display_walls", "a")
-
     def test_setting_all_short_connectors(self):
         po = ParseOptions()
         po.all_short_connectors = True
@@ -197,7 +153,7 @@ class ParseOptionsTestCase(unittest.TestCase):
         self.assertRaises(TypeError, setattr, po, "all_short_connectors", "a")
 
 
-class ParserTestCase(unittest.TestCase):
+class CParserTestCase(unittest.TestCase):
     def test_specifying_options_when_instantiating_parser(self):
         p = Parser(linkage_limit=10)
         self.assertEqual(clg.parse_options_get_linkage_limit(p.parse_options._po), 10)
@@ -215,8 +171,7 @@ class ParserTestCase(unittest.TestCase):
         del p
 
 
-
-class ParsingTestCase(unittest.TestCase):
+class DBasicParsingTestCase(unittest.TestCase):
     def setUp(self):
         self.p = Parser()
 
@@ -265,9 +220,10 @@ class ParsingTestCase(unittest.TestCase):
         result = self.p.parse_sent("This is a sentence.")[0]
         self.assertEqual(result.link_distances, [5,2,1,1,2,1,1])
         result = self.p.parse_sent("This is a silly sentence.")[0]
-        self.assertEqual(result.link_distances, [6,2,1,1,3,2,1,1])
+        self.assertEqual(result.link_distances, [6,2,1,1,3,2,1,1,1])
 
-class LinkageTestCase(unittest.TestCase):
+
+class EEnglishLinkageTestCase(unittest.TestCase):
     def setUp(self):
         self.p = Parser()
 
@@ -285,7 +241,7 @@ class LinkageTestCase(unittest.TestCase):
         self.assertEqual(linkage.links[0],
                          Link('LEFT-WALL','Xp','Xp','.'))
         self.assertEqual(linkage.links[1],
-                         Link('LEFT-WALL','WV','WV','is.v'))
+                         Link('LEFT-WALL','hWV','dWV','is.v'))
         self.assertEqual(linkage.links[2],
                          Link('LEFT-WALL','Wd','Wd','this.p'))
         self.assertEqual(linkage.links[3],
@@ -293,17 +249,18 @@ class LinkageTestCase(unittest.TestCase):
         self.assertEqual(linkage.links[4],
                          Link('is.v','O*m','Os','sentence.n'))
         self.assertEqual(linkage.links[5],
-                         Link('a','Ds','Ds','sentence.n'))
+                         Link('a','Ds**c','Ds**c','sentence.n'))
         self.assertEqual(linkage.links[6],
                          Link('.','RW','RW','RIGHT-WALL'))
 
     def test_d_spell_guessing_on(self):
         self.p = Parser(spell_guess = True)
         result = self.p.parse_sent("I love going to shoop.")
+        resultx = result[0] if result else []
         for resultx in result:
             if resultx.words[5] == 'shop[~].v':
                 break;
-        self.assertEqual(resultx.words,
+        self.assertEqual(resultx.words if resultx else [],
              ['LEFT-WALL', 'I.p', 'love.v', 'going.v', 'to.r', 'shop[~].v', '.', 'RIGHT-WALL'])
 
     def test_e_spell_guessing_off(self):
@@ -344,15 +301,17 @@ class LinkageTestCase(unittest.TestCase):
              ['LEFT-WALL', 'Jumbo[!]', 'sat.v-d', 'down.r', '.', 'RIGHT-WALL'])
 
         # Red is in dict, lower-case, as noun, too.
-        self.assertEqual(self.p.parse_sent('Red\'s going?')[0].words,
-             ['LEFT-WALL', 'Red[!]', '\'s.v', 'going.v', '?', 'RIGHT-WALL'])
-
-        self.assertEqual(self.p.parse_sent('Red\'s shoe fell off.')[0].words,
-             ['LEFT-WALL', 'Red[!]',
-              '\'s.p', 'shoe.n', 'fell.v-d', 'off', '.', 'RIGHT-WALL'])
-
-        self.assertEqual(self.p.parse_sent('Red sat down.')[1].words,
-             ['LEFT-WALL', 'Red[!]', 'sat.v-d', 'down.r', '.', 'RIGHT-WALL'])
+        # There's no way to really know, syntactically, that Red 
+        # should be taken as a proper noun (given name).
+        #self.assertEqual(self.p.parse_sent('Red\'s going?')[0].words,
+        #     ['LEFT-WALL', 'Red[!]', '\'s.v', 'going.v', '?', 'RIGHT-WALL'])
+        #
+        #self.assertEqual(self.p.parse_sent('Red\'s shoe fell off.')[0].words,
+        #     ['LEFT-WALL', 'Red[!]',
+        #      '\'s.p', 'shoe.n', 'fell.v-d', 'off', '.', 'RIGHT-WALL'])
+        #
+        #self.assertEqual(self.p.parse_sent('Red sat down.')[1].words,
+        #     ['LEFT-WALL', 'Red[!]', 'sat.v-d', 'down.r', '.', 'RIGHT-WALL'])
 
         # May in dict as noun, capitalized, and as lower-case verb.
         self.assertEqual(self.p.parse_sent('May\'s going?')[0].words,
@@ -391,18 +350,81 @@ class LinkageTestCase(unittest.TestCase):
              ['LEFT-WALL', 'Thomas.b', 'Industries[!]',
               'stock.n-u', 'declined.v-d', '.', 'RIGHT-WALL'])
 
+    # Some parses are fractionally preferred over others...
+    def test_g_fractions(self):
+        self.assertEqual(self.p.parse_sent('A player who is injured has to leave the field')[0].words,
+             ['LEFT-WALL', 'a', 'player.n', 'who', 'is.v', 'injured.a', 'has.v', 'to.r', 'leave.v', 'the', 'field.n', 'RIGHT-WALL'])
 
-class LinkTestCase(unittest.TestCase):
-    def test_link_display_with_identical_link_type(self):
-        self.assertEqual(unicode(Link('Left','Link','Link','Right')),
-                         u'Left-Link-Right')
+        self.assertEqual(self.p.parse_sent('They ate a special curry which was recommended by the restaurant\'s owner')[0].words,
+             ['LEFT-WALL', 'they', 'ate.v-d', 'a', 'special.a', 'curry.s',
+              'which', 'was.v-d', 'recommended.v-d', 'by', 'the', 'restaurant.n',
+              '\'s.p', 'owner.n', 'RIGHT-WALL'])
+
+    # Verify that we are getting the linkages that we want
+    # See below, remainder of parses are in text files
+    def test_h_getting_links(self):
+        sent = 'Scientists sometimes may repeat experiments or use groups.'
+        linkage = self.p.parse_sent(sent)[0]
+        self.assertEqual(linkage.diagram, 
+"\n    +---------------------------------------Xp--------------------------------------+"
+"\n    +---------------------------->WV---------------------------->+                  |"
+"\n    |           +-----------------------Sp-----------------------+                  |"
+"\n    |           |                  +------------VJlpi------------+                  |"
+"\n    +-----Wd----+          +---E---+---I---+----Op----+          +VJrpi+---Op--+    |"
+"\n    |           |          |       |       |          |          |     |       |    |"
+"\nLEFT-WALL scientists.n sometimes may.v repeat.v experiments.n or.j-v use.v groups.n . "
+"\n\n")
+        sent = 'I enjoy eating bass.'
+        linkage = self.p.parse_sent(sent)[0]
+        self.assertEqual(linkage.diagram, 
+"\n    +-----------------Xp----------------+"
+"\n    +---->WV---->+                      |"
+"\n    +--Wd--+-Sp*i+---Pg---+---Ou---+    |"
+"\n    |      |     |        |        |    |"
+"\nLEFT-WALL I.p enjoy.v eating.v bass.n-u . "
+"\n\n")
+
+
+        sent = 'We are from the planet Gorpon'
+        linkage = self.p.parse_sent(sent)[0]
+        self.assertEqual(linkage.diagram, 
+"\n    +--->WV--->+     +---------Js--------+"
+"\n    +--Wd--+Spx+--Pp-+   +--DD--+---GN---+"
+"\n    |      |   |     |   |      |        |"
+"\nLEFT-WALL we are.v from the planet.n Gorpon[!] "
+"\n\n")
+
+
+class ZENLangTestCase(unittest.TestCase):
+    def setUp(self):
+        self.p = Parser(lang = 'en')
+
+    # Reads linkages from a test-file.
+    def test_getting_links(self):
+        parses = open("parses-en.txt")
+        diagram = None
+        sent = None
+        for line in parses :
+            # Lines starting with I are the input sentences
+            if 'I' == line[0] :
+                sent = line[1:]
+                diagram = ""
+
+            # Lines starting with O are the parse diagrams
+            if 'O' == line[0] :
+                diagram += line[1:]
+
+                # We have a complete diagram if it ends with an
+                # empty line.
+                if '\n' == line[1] and 1 < len(diagram) :
+                    linkage = self.p.parse_sent(sent)[0]
+                    self.assertEqual(linkage.diagram, diagram)
+
+        parses.close()
 
-    def test_link_display_with_identical_link_type(self):
-        self.assertEqual(unicode(Link('Left','Link','Link*','Right')),
-                         u'Left-Link-Link*-Right')
 
 # Tests are run in alphabetical order; do the language tests last.
-class ZLangTestCaseDE(unittest.TestCase):
+class ZDELangTestCase(unittest.TestCase):
     def setUp(self):
         self.p = Parser(lang = 'de')
 
@@ -432,8 +454,36 @@ class ZLangTestCaseDE(unittest.TestCase):
         self.assertEqual(linkage.links[5],
                          Link('.','RW','RW','RIGHT-WALL'))
 
+class ZLTLangTestCase(unittest.TestCase):
+    def setUp(self):
+        self.p = Parser(lang = 'lt')
+
+    # Reads linkages from a test-file.
+    def test_getting_links(self):
+        parses = open("parses-lt.txt")
+        diagram = None
+        sent = None
+        for line in parses :
+            # Lines starting with I are the input sentences
+            if 'I' == line[0] :
+                sent = line[1:]
+                diagram = ""
+
+            # Lines starting with O are the parse diagrams
+            if 'O' == line[0] :
+                diagram += line[1:]
+
+                # We have a complete diagram if it ends with an
+                # empty line.
+                if '\n' == line[1] and 1 < len(diagram) :
+                    linkage = self.p.parse_sent(sent)[0]
+                    self.assertEqual(linkage.diagram, diagram)
+
+        parses.close()
+
+
 # Tests are run in alphabetical order; do the language tests last.
-class ZLangTestCaseRU(unittest.TestCase):
+class ZRULangTestCase(unittest.TestCase):
     def setUp(self):
         self.p = Parser(lang = 'ru')
 
