@@ -125,6 +125,11 @@ const char * linkgrammar_get_dict_version(Dictionary dict)
   These costs are called "disjunct costs".  The linkages are printed out
   in order of non-increasing disjunct cost.
 
+  A number following a square bracket over-rides the cost of that bracket.
+  Thus, [...].5 has a cost of 0.5 while [...]2.0 has a cost of 2; that
+  is it is the same as [[...]].  Any floating point number (including
+  exponents!) is allowed.
+
   The expression "(A+ or ())" means that you can choose either "A+" or
   the empty expression "()", that is, that the connector "A+" is
   optional.  This is more compactly expressed as "{A+}".  In other words,
@@ -149,7 +154,7 @@ const char * linkgrammar_get_dict_version(Dictionary dict)
   to distinguish different word senses.
 */
 
-static Boolean link_advance(Dictionary dict);
+static bool link_advance(Dictionary dict);
 
 static void dict_error2(Dictionary dict, const char * s, const char *s2)
 {
@@ -161,7 +166,7 @@ static void dict_error2(Dictionary dict, const char * s, const char *s2)
 	/* The link_advance used to print the error message can
 	 * throw more errors while printing... */
 	if (dict->recursive_error) return;
-	dict->recursive_error = TRUE;
+	dict->recursive_error = true;
 
 	tokens[0] = '\0';
 	for (i=0; i<5 && dict->token[0] != '\0' ; i++)
@@ -190,7 +195,7 @@ static void dict_error2(Dictionary dict, const char * s, const char *s2)
 		        dict->name,
 		        s, dict->line_number, tokens);
 	}
-	dict->recursive_error = FALSE;
+	dict->recursive_error = false;
 }
 
 static void dict_error(Dictionary dict, const char * s)
@@ -284,7 +289,7 @@ static utf8char get_character(Dictionary dict, int quote_mode)
  * Return true if the input character is one of the special
  * characters used to define the syntax of the dictionary.
  */
-static Boolean char_is_special(char c)
+static bool char_is_special(char c)
 {
 	return (NULL != strchr(SPECIAL, c));
 }
@@ -293,13 +298,13 @@ static Boolean char_is_special(char c)
  * This reads the next token from the input into 'token'.
  * Return 1 if a character was read, else return 0 (and print a warning).
  */
-static Boolean link_advance(Dictionary dict)
+static bool link_advance(Dictionary dict)
 {
 	utf8char c;
 	int nr, i;
 	int quote_mode;
 
-	dict->is_special = FALSE;
+	dict->is_special = false;
 
 	if (dict->already_got_it != '\0')
 	{
@@ -311,19 +316,19 @@ static Boolean link_advance(Dictionary dict)
 			dict->token[1] = '\0';
 		}
 		dict->already_got_it = '\0';
-		return TRUE;
+		return true;
 	}
 
-	do { c = get_character(dict, FALSE); } while (lg_isspace(c[0]));
+	do { c = get_character(dict, false); } while (lg_isspace(c[0]));
 
-	quote_mode = FALSE;
+	quote_mode = false;
 
 	i = 0;
 	for (;;)
 	{
 		if (i > MAX_TOKEN_LENGTH-3) {
 			dict_error(dict, "Token too long");
-			return FALSE;
+			return false;
 		}
 		if (quote_mode) {
 			if (c[0] == '\"' &&
@@ -331,13 +336,13 @@ static Boolean link_advance(Dictionary dict)
 			    (*dict->pin == ':' || *dict->pin == ';' ||
 			    lg_isspace(*dict->pin))) {
 
-				quote_mode = FALSE;
+				quote_mode = false;
 				dict->token[i] = '\0';
-				return TRUE;
+				return true;
 			}
 			if (lg_isspace(c[0])) {
 				dict_error(dict, "White space inside of token");
-				return FALSE;
+				return false;
 			}
 
 			nr = 0;
@@ -349,28 +354,28 @@ static Boolean link_advance(Dictionary dict)
 				{
 					dict->token[0] = c[0];  /* special toks are one char always */
 					dict->token[1] = '\0';
-					dict->is_special = TRUE;
-					return TRUE;
+					dict->is_special = true;
+					return true;
 				}
 				dict->token[i] = '\0';
 				dict->already_got_it = c[0];
-				return TRUE;
+				return true;
 			}
 			if (c[0] == 0x0) {
 				if (i == 0) {
 					dict->token[0] = '\0';
-					return TRUE;
+					return true;
 				}
 				dict->token[i] = '\0';
 				dict->already_got_it = '\0';
-				return TRUE;
+				return true;
 			}
 			if (lg_isspace(c[0])) {
 				dict->token[i] = '\0';
-				return TRUE;
+				return true;
 			}
 			if (c[0] == '\"') {
-				quote_mode = TRUE;
+				quote_mode = true;
 			} else {
 				nr = 0;
 				while (c[nr]) {dict->token[i] = c[nr]; i++; nr++; }
@@ -378,11 +383,11 @@ static Boolean link_advance(Dictionary dict)
 		}
 		c = get_character(dict, quote_mode);
 	}
-	return TRUE;
+	return true;
 }
 
 /**
- * Returns TRUE if this token is a special token and it is equal to c
+ * Returns true if this token is a special token and it is equal to c
  */
 static int is_equal(Dictionary dict, char c)
 {
@@ -393,39 +398,39 @@ static int is_equal(Dictionary dict, char c)
 
 /**
  * Make sure the string s is a valid connector.
- * Return TRUE if the connector is valid, else return FALSE,
+ * Return true if the connector is valid, else return false,
  * and print an appropriate warning message.
  */
-static Boolean check_connector(Dictionary dict, const char * s)
+static bool check_connector(Dictionary dict, const char * s)
 {
 	int i;
 	i = strlen(s);
 	if (i < 1) {
 		dict_error(dict, "Expecting a connector.");
-		return FALSE;
+		return false;
 	}
 	i = s[i-1];  /* the last character of the token */
 	if ((i != '+') && (i != '-') && (i != ANY_DIR)) {
-		dict_error(dict, "A connector must end in a \"+\", \"-\" or \"^\".");
-		return FALSE;
+		dict_error(dict, "A connector must end in a \"+\", \"-\" or \"$\".");
+		return false;
 	}
 	if (*s == '@') s++;
-	if (!isupper((int)*s)) {
-		dict_error(dict, "The first letter of a connector must be in [A--Z].");
-		return FALSE;
+	if (!isupper((int)*s) && ('h' != *s) && ('d' != *s)) {
+		dict_error(dict, "The first letter of a connector must be h,d or uppercase.");
+		return false;
 	}
 	if ((*s == 'I') && (*(s+1) == 'D')) {
 		dict_error(dict, "Connectors beginning with \"ID\" are forbidden");
-		return FALSE;
+		return false;
 	}
 	while (*(s+1)) {
 		if ((!isalnum((int)*s)) && (*s != WILD_TYPE)) {
 			dict_error(dict, "All letters of a connector must be ASCII alpha-numeric.");
-			return FALSE;
+			return false;
 		}
 		s++;
 	}
-	return TRUE;
+	return true;
 }
 
 /* ======================================================================== */
@@ -542,19 +547,11 @@ static inline int dict_order_wild(const char * s, Dict_node * dn)
 	lgdebug(+5, "search-word='%s' dict-word='%s'\n", s, t);
 	while((*s != '\0') && (*s != SUBSCRIPT_MARK) && (*s == *t)) {s++; t++;}
 
-	if (*s == WILD_TYPE)
-	{
-		/* Skip the rest of t */
-		s++;
-		while((*t != '\0') && (*t != SUBSCRIPT_MARK)) t++;
-	}
-
-	lgdebug(5, "Rest: '%s' '%s'\n", s, t);
-	while(*s != '\0' && *s == *t) {s++; t++;}
+	if (*s == WILD_TYPE) return 0;
 
 	lgdebug(5, "Result: '%s'-'%s'=%d\n",
-	 s, t, (*s) - ((*t == SUBSCRIPT_MARK)?(0):(*t)));
-	return (*s) - ((*t == SUBSCRIPT_MARK)?(0):(*t));
+	 s, t, ((*s == SUBSCRIPT_MARK)?(0):(*s)) - ((*t == SUBSCRIPT_MARK)?(0):(*t)));
+	return ((*s == SUBSCRIPT_MARK)?(0):(*s)) - ((*t == SUBSCRIPT_MARK)?(0):(*t));
 }
 
 /**
@@ -565,7 +562,7 @@ static inline int dict_order_wild(const char * s, Dict_node * dn)
  *
  * A subscript is the part that follows the SUBSCRIPT_MARK.
  */
-static Boolean dict_match(const char * s, const char * t)
+static bool dict_match(const char * s, const char * t)
 {
 	char *ds, *dt;
 	ds = strrchr(s, SUBSCRIPT_MARK);
@@ -582,10 +579,10 @@ static Boolean dict_match(const char * s, const char * t)
 
 	/* dt is NULL when there's no prefix ... */
 	if (dt == NULL && ds != NULL) {
-		if (((int)strlen(t)) > (ds-s)) return FALSE;   /* we need to do this to ensure that */
+		if (((int)strlen(t)) > (ds-s)) return false;   /* we need to do this to ensure that */
 		return (strncmp(s, t, ds-s) == 0);	           /* "i.e." does not match "i.e" */
 	} else if (dt != NULL && ds == NULL) {
-		if (((int)strlen(s)) > (dt-t)) return FALSE;
+		if (((int)strlen(s)) > (dt-t)) return false;
 		return (strncmp(s, t, dt-t) == 0);
 	} else {
 		return (strcmp(s, t) == 0);
@@ -642,6 +639,19 @@ static Dict_node * prune_lookup_list(Dict_node *llist, const char * s)
 }
 
 /* ======================================================================== */
+static bool subscr_match(const char *s, Dict_node * dn)
+{
+	const char * s_sub = strrchr(s, SUBSCRIPT_MARK);
+	const char * t_sub;
+
+	if (NULL == s_sub) return true;
+	t_sub = strrchr(dn->string, SUBSCRIPT_MARK);
+	if (NULL == t_sub) return false;
+	if ( 0 == strcmp(s_sub, t_sub)) return true;
+
+	return false;
+}
+
 /**
  * rdictionary_lookup() -- recursive dictionary lookup
  * Walk binary tree, given by 'dn', looking for the string 's'.
@@ -653,23 +663,21 @@ rdictionary_lookup(Dict_node *llist,
                    Dict_node * dn,
                    const char * s,
                    bool match_idiom,
-                   bool wild_lookup)
+                   int (*dict_order)(const char *, Dict_node *))
 {
 	/* see comment in dictionary_lookup below */
 	int m;
 	Dict_node * dn_new;
 	if (dn == NULL) return llist;
 
-	if (wild_lookup)
-		m = dict_order_wild(s, dn);
-	else
-		m = dict_order_bare(s, dn);
+	m = dict_order(s, dn);
 
 	if (m >= 0)
 	{
-		llist = rdictionary_lookup(llist, dn->right, s, match_idiom, wild_lookup);
+		llist = rdictionary_lookup(llist, dn->right, s, match_idiom, dict_order);
 	}
-	if ((m == 0) && (match_idiom || !is_idiom_word(dn->string)))
+	if ((m == 0) && (match_idiom || !is_idiom_word(dn->string)) &&
+		 (dict_order != dict_order_wild || subscr_match(s, dn)))
 	{
 		dn_new = dict_node_new();
 		*dn_new = *dn;
@@ -678,7 +686,7 @@ rdictionary_lookup(Dict_node *llist,
 	}
 	if (m <= 0)
 	{
-		llist = rdictionary_lookup(llist, dn->left, s, match_idiom, wild_lookup);
+		llist = rdictionary_lookup(llist, dn->left, s, match_idiom, dict_order);
 	}
 	return llist;
 }
@@ -697,15 +705,16 @@ rdictionary_lookup(Dict_node *llist,
  */
 Dict_node * lookup_list(Dictionary dict, const char *s)
 {
-	Dict_node * llist = rdictionary_lookup(NULL, dict->root, s, TRUE, FALSE);
+	Dict_node * llist =
+		rdictionary_lookup(NULL, dict->root, s, true, dict_order_bare);
 	llist = prune_lookup_list(llist, s);
 	return llist;
 }
 
-Boolean boolean_lookup(Dictionary dict, const char *s)
+bool boolean_lookup(Dictionary dict, const char *s)
 {
 	Dict_node *llist = lookup_list(dict, s);
-	Boolean boool = (llist != NULL);
+	bool boool = (llist != NULL);
 	free_lookup(llist);
 	return boool;
 }
@@ -736,7 +745,8 @@ static Dict_node * dictionary_lookup_wild(Dictionary dict, const char *s)
 	if ((NULL != ds) && ('\0' != ds[1]) && ((NULL == ws) || (ds > ws)))
 		stmp[ds-s] = SUBSCRIPT_MARK;
 
-	result = rdictionary_lookup(NULL, dict->root, stmp, lookup_idioms, TRUE);
+	result =
+	 rdictionary_lookup(NULL, dict->root, stmp, lookup_idioms, dict_order_wild);
 	free(stmp);
 	return result;
 }
@@ -756,7 +766,7 @@ static Dict_node * dictionary_lookup_wild(Dictionary dict, const char *s)
 Dict_node * abridged_lookup_list(Dictionary dict, const char *s)
 {
 	Dict_node *llist;
-	llist = rdictionary_lookup(NULL, dict->root, s, FALSE, FALSE);
+	llist = rdictionary_lookup(NULL, dict->root, s, false, dict_order_bare);
 	llist = prune_lookup_list(llist, s);
 	return llist;
 }
@@ -877,12 +887,12 @@ static Exp * make_dir_connector(Dictionary dict, int i)
 	if (dict->token[0] == '@')
 	{
 		n->u.string = string_set_add(dict->token+1, dict->string_set);
-		n->multi = TRUE;
+		n->multi = true;
 	}
 	else
 	{
 		n->u.string = string_set_add(dict->token, dict->string_set);
-		n->multi = FALSE;
+		n->multi = false;
 	}
 	n->type = CONNECTOR_type;
 	n->cost = 0.0;
@@ -1003,13 +1013,17 @@ void add_empty_word(Dictionary dict, Dict_node * dn)
 	size_t len;
 	Exp *zn, *an;
 	E_list *elist, *flist;
+	/* We assume the affix file has been read by now, so INFIX_MARK is set */
+	Dictionary afdict = dict->affix_table; /* for INFIX_MARK only */
+	char infix_mark = INFIX_MARK;
 
 	if (! dict->empty_word_defined) return;
 
-	if (STEM_MARK == dn->string[0]) return;
+	if (infix_mark == dn->string[0]) return;
 
 	len = strlen(dn->string);
 	if (STEM_MARK == dn->string[len-1]) return;
+	if (infix_mark == dn->string[len-1]) return;
 	if (0 == strcmp(dn->string, LEFT_WALL_WORD)) return;
 	if (0 == strcmp(dn->string, RIGHT_WALL_WORD)) return;
 
@@ -1021,7 +1035,7 @@ void add_empty_word(Dictionary dict, Dict_node * dn)
 	zn = Exp_create(dict);
 	zn->dir = '+';
 	zn->u.string = string_set_add(EMPTY_CONNECTOR, dict->string_set);
-	zn->multi = FALSE;
+	zn->multi = false;
 	zn->type = CONNECTOR_type;
 	zn->cost = 0.0;
 	zn = make_optional_node(dict, zn);
@@ -1048,6 +1062,17 @@ void add_empty_word(Dictionary dict, Dict_node * dn)
 
 /* ======================================================================== */
 
+/** Return true if the string is a (floating point) number */
+static bool is_number(const char * str)
+{
+	if (strspn(str, "0123456789.") == strlen(str))
+		return true;
+
+	return false;
+}
+
+/* ======================================================================== */
+
 /* INFIX_NOTATION is always defined; we simply never use the format below. */
 /* #if ! defined INFIX_NOTATION */
 #if 0
@@ -1180,7 +1205,7 @@ static Exp * restricted_expression(Dictionary dict, int and_ok, int or_ok);
  */
 static Exp * make_expression(Dictionary dict)
 {
-	return restricted_expression(dict, TRUE, TRUE);
+	return restricted_expression(dict, true, true);
 }
 
 static Exp * restricted_expression(Dictionary dict, int and_ok, int or_ok)
@@ -1238,7 +1263,23 @@ static Exp * restricted_expression(Dictionary dict, int and_ok, int or_ok)
 		if (!link_advance(dict)) {
 			return NULL;
 		}
-		nl->cost += 1.0;
+
+		/* A square bracket can have a number after it.
+		 * If it is present, then that number is interpreted
+		 * as the cost of the bracket. Else, the cost of a
+		 * square bracket is 1.0.
+		 */
+		if (is_number(dict->token))
+		{
+			nl->cost += atof(dict->token);
+			if (!link_advance(dict)) {
+				return NULL;
+			}
+		}
+		else
+		{
+			nl->cost += 1.0;
+		}
 	}
 	else if (!dict->is_special)
 	{
@@ -1269,7 +1310,7 @@ static Exp * restricted_expression(Dictionary dict, int and_ok, int or_ok)
 		if (!link_advance(dict)) {
 			return NULL;
 		}
-		nr = restricted_expression(dict, TRUE, FALSE);
+		nr = restricted_expression(dict, true, false);
 		if (nr == NULL) {
 			return NULL;
 		}
@@ -1286,7 +1327,7 @@ static Exp * restricted_expression(Dictionary dict, int and_ok, int or_ok)
 		if (!link_advance(dict)) {
 			return NULL;
 		}
-		nr = restricted_expression(dict, FALSE, TRUE);
+		nr = restricted_expression(dict, false, true);
 		if (nr == NULL) {
 			return NULL;
 		}
@@ -1303,7 +1344,7 @@ static Exp * restricted_expression(Dictionary dict, int and_ok, int or_ok)
 		if (!link_advance(dict)) {
 			return NULL;
 		}
-		nr = restricted_expression(dict, TRUE, FALSE);
+		nr = restricted_expression(dict, true, false);
 		if (nr == NULL) {
 			return NULL;
 		}
@@ -1549,7 +1590,7 @@ void insert_list(Dictionary dict, Dict_node * p, int l)
  * and is terminated by a semi-colon.
  * Add these words to the dictionary.
  */
-static Boolean read_entry(Dictionary dict)
+static bool read_entry(Dictionary dict)
 {
 	Exp *n;
 	int i;
@@ -1575,16 +1616,16 @@ static Boolean read_entry(Dictionary dict)
 				err_ctxt ec;
 				ec.sent = NULL;
 				err_msg(&ec, Error, "Error opening word file %s", dict->token);
-				return FALSE;
+				return false;
 			}
 		}
 		else if ((dict->token[0] == '#') && (0 == strcmp(dict->token, "#include")))
 		{
-			Boolean rc;
+			bool rc;
 			char* instr;
 			char* dict_name;
 			const char * save_name;
-			Boolean save_is_special;
+			bool save_is_special;
 			const char * save_input;
 			const char * save_pin;
 			char save_already_got_it;
@@ -1637,7 +1678,7 @@ static Boolean read_entry(Dictionary dict)
 				if (!link_advance(dict)) goto syntax_error;
 			}
 
-			return TRUE;
+			return true;
 		}
 		else
 		{
@@ -1648,6 +1689,8 @@ static Boolean read_entry(Dictionary dict)
 			dn = dn_new;
 			dn->file = NULL;
 
+			/* Note: The follwoing patches a dot in regexes appearing in
+			 * the affix file... It is corrected later. */
 			patch_subscript(dict->token);
 			dn->string = string_set_add(dict->token, dict->string_set);
 		}
@@ -1689,11 +1732,11 @@ static Boolean read_entry(Dictionary dict)
 		i++;
 	}
 	dict->insert_entry(dict, dn, i);
-	return TRUE;
+	return true;
 
 syntax_error:
 	free_lookup(dn);
-	return FALSE;
+	return false;
 }
 
 
@@ -1717,22 +1760,26 @@ void print_dictionary_data(Dictionary dict)
 	rprint_dictionary_data(dict->root);
 }
 
-Boolean read_dictionary(Dictionary dict)
+bool read_dictionary(Dictionary dict)
 {
 	if (!link_advance(dict))
 	{
-		return FALSE;
+		return false;
 	}
-	while (dict->token[0] != '\0')
+	/* The last character of a dictionary is NUL.
+	 * Note: At the end of reading a dictionary, dict->pin points to one
+	 * character after the input. Referring its [-1] element is safe even if
+	 * the dict file size is 0. */
+	while ('\0' != dict->pin[-1])
 	{
 		if (!read_entry(dict))
 		{
-			return FALSE;
+			return false;
 		}
 	}
 	dict->root = dsw_tree_to_vine(dict->root);
 	dict->root = dsw_vine_to_tree(dict->root, dict->num_entries);
-	return TRUE;
+	return true;
 }
 
 /* ======================================================================= */
@@ -1755,14 +1802,14 @@ static void display_word_split(Dictionary dict,
 	Sentence sent;
 	struct Parse_Options_s display_word_opts = *opts;
 
-	parse_options_set_spell_guess(&display_word_opts, 0);
+	parse_options_set_spell_guess(&display_word_opts, false);
 	sent = sentence_create(word, dict);
 	separate_sentence(sent, &display_word_opts);
 
 	/* List the splits */
-	print_sentence_word_alternatives(sent, FALSE, NULL, NULL);
+	print_sentence_word_alternatives(sent, false, NULL, NULL);
 	/* List the disjuncts information */
-	print_sentence_word_alternatives(sent, FALSE, display, NULL);
+	print_sentence_word_alternatives(sent, false, display, NULL);
 
 	sentence_delete(sent);
 }
@@ -1833,7 +1880,7 @@ static void display_word_info(Dictionary dict, const char * word)
 	}
 
 	/* Recurse, if its a regex match */
-	regex_name = match_regex(dict, word);
+	regex_name = match_regex(dict->regex_root, word);
 	if (regex_name)
 	{
 		display_word_info(dict, regex_name);
@@ -1856,7 +1903,7 @@ static void display_word_expr(Dictionary dict, const char * word)
 	}
 
 	/* Recurse, if its a regex match */
-	regex_name = match_regex(dict, word);
+	regex_name = match_regex(dict->regex_root, word);
 	if (regex_name)
 	{
 		display_word_expr(dict, regex_name);
