@@ -1,11 +1,11 @@
 /*************************************************************************/
 /* Copyright (c) 2004                                                    */
 /* Daniel Sleator, David Temperley, and John Lafferty                    */
+/* Copyright (c) 2013 Linas Vepstas                                      */
 /* All rights reserved                                                   */
 /*                                                                       */
 /* Use of the link grammar parsing system is subject to the terms of the */
-/* license set forth in the LICENSE file included with this software,    */
-/* and also available at http://www.link.cs.cmu.edu/link/license.html    */
+/* license set forth in the LICENSE file included with this software.    */
 /* This license allows free redistribution and use in source and binary  */
 /* forms, with or without modification, subject to certain conditions.   */
 /*                                                                       */
@@ -44,14 +44,14 @@ typedef enum
 struct Exp_struct
 {
     Exp * next;    /* Used only for mem management,for freeing */
-    Exp_type type; /* One of three types, see above */
+    Exp_type type; /* One of three types: AND, OR, or connector. */
     char dir;      /* '-' means to the left, '+' means to right (for connector) */
-    Boolean multi; /* TRUE if a multi-connector (for connector)  */
+    bool multi;    /* TRUE if a multi-connector (for connector)  */
     union {
         E_list * l;           /* Only needed for non-terminals */
         const char * string;  /* Only needed if it's a connector */
     } u;
-    float cost;    /* The cost of using this expression.
+    double cost;   /* The cost of using this expression.
                       Only used for non-terminals */
 };
 
