@@ -283,7 +283,7 @@ link_public_api(char *)
 link_public_api(void)
      linkage_free_diagram(char * str);
 link_public_api(char *)
-     linkage_print_postscript(Linkage linkage, bool display_walls, bool print_ps_header);
+     linkage_print_postscript(const Linkage linkage, bool display_walls, bool print_ps_header);
 link_public_api(void)
      linkage_free_postscript(char * str);
 link_public_api(char *)
@@ -343,7 +343,7 @@ link_public_api(void)
      dict_display_word_info(Dictionary dict, const char *, Parse_Options opts);
 link_public_api(void)
      left_print_string(FILE* fp, const char *, const char *);
-link_public_api(int)
+link_public_api(bool)
      lg_expand_disjunct_list(Sentence sent);
 
 
@@ -446,10 +446,6 @@ MS_DEPRECATED link_public_api(void)
      parse_options_set_use_fat_links(Parse_Options opts, int use_fat_links) GNUC_DEPRECATED;
 MS_DEPRECATED link_public_api(int)
      parse_options_get_use_fat_links(Parse_Options opts) GNUC_DEPRECATED;
-MS_DEPRECATED link_public_api(void)
-     parse_options_set_display_union(Parse_Options opts, int val) GNUC_DEPRECATED;
-MS_DEPRECATED link_public_api(int)
-     parse_options_get_display_union(Parse_Options opts) GNUC_DEPRECATED;
 
 /**********************************************************************
  *
