@@ -38,9 +38,9 @@ struct disjunct_dup_table_s
 typedef struct c_list_s C_list;
 struct c_list_s
 {
-	Connector * c;
-	Boolean shallow;
 	C_list * next;
+	Connector * c;
+	bool shallow;
 };
 
 typedef struct power_table_s power_table;
@@ -71,7 +71,7 @@ struct multiset_table_s
 typedef struct prune_context_s prune_context;
 struct prune_context_s
 {
-	Boolean null_links;
+	bool null_links;
 #ifdef USE_FAT_LINKAGES
 	char ** deletable;
 	char ** effective_dist;
@@ -148,26 +148,26 @@ static inline unsigned int hash_S(Connector * c)
  * We assume that a is on a word to the left of b.
  */
 #ifdef USE_FAT_LINKAGES
-int prune_match(int dist, Connector *a, Connector *b)
+bool prune_match(int dist, Connector *a, Connector *b)
 {
 	const char *s, *t;
 	int x, y;
 
-	if (a->label != b->label) return FALSE;
+	if (a->label != b->label) return false;
 
 	s = a->string;
 	t = b->string;
 
 	while (isupper((int)*s) || isupper((int)*t))
 	{
-		if (*s != *t) return FALSE;
+		if (*s != *t) return false;
 		s++;
 		t++;
 	}
 
 	/*	printf("PM: a=%4s b=%4s  ap=%d bp=%d  a->ll=%d b->ll=%d  dist=%d\n",
 	   s, t, x, y, a->length_limit, b->length_limit, dist); */
-	if (dist > a->length_limit || dist > b->length_limit) return FALSE;
+	if (dist > a->length_limit || dist > b->length_limit) return false;
 
 	x = a->priority;
 	y = b->priority;
@@ -190,7 +190,7 @@ int prune_match(int dist, Connector *a, Connector *b)
 		if ((*(a->string)=='S') && ((*s=='s') || (*s=='p')) &&
 			((*t=='p') || (*t=='s')) &&
 			((s-1 == a->string) || ((s-2 == a->string) && (*(s-1) == 'I')))){
-			return TRUE;
+			return true;
 		}
 /*
    This change is to accommodate "nor".  In particular we need to
@@ -212,9 +212,9 @@ int prune_match(int dist, Connector *a, Connector *b)
 				t++;
 			}
 			else
-				return FALSE;
+				return false;
 		}
-		return TRUE;
+		return true;
 	}
 	else if ((x == UP_priority) && (y == DOWN_priority))
 	{
@@ -228,9 +228,9 @@ int prune_match(int dist, Connector *a, Connector *b)
 				t++;
 			}
 			else
-				return FALSE;
+				return false;
 		}
-		return TRUE;
+		return true;
 	}
 	else if ((y == UP_priority) && (x == DOWN_priority))
 	{
@@ -242,12 +242,12 @@ int prune_match(int dist, Connector *a, Connector *b)
 				t++;
 			}
 			else
-				return FALSE;
+				return false;
 		}
-		return TRUE;
+		return true;
 	}
 	else
-		return FALSE;
+		return false;
 }
 
 #else /* not USE_FAT_LINKAGES */
@@ -513,33 +513,33 @@ static int string_hash(disjunct_dup_table *dt, const char * s, int i)
  * remains TRUE if multi-match added to the first.
  * remains TRUE if subsrcripts deleted from the first.
  */
-int connector_matches_alam(Connector * a, Connector * b)
+static bool connector_matches_alam(Connector * a, Connector * b)
 {
 	char * s, * t, *u;
 	if (((!a->multi) && b->multi) ||
 		(a->label != b->label) ||
-		(a->priority != b->priority))  return FALSE;
+		(a->priority != b->priority))  return false;
 	s = a->string;
 	t = b->string;
 
 	/* isupper -- connectors cannot be UTF8 at this time */
-	while(isupper(*s) || isupper(*t))
+	while (isupper(*s) || isupper(*t))
 	{
 		if (*s == *t) {
 			s++;
 			t++;
-		} else return FALSE;
+		} else return false;
 	}
 	if (a->priority == DOWN_priority) {
 		u = s;
 		s = t;
 		t = u;
 	}
-	while((*s != '\0') && (*t != '\0')) {
+	while ((*s != '\0') && (*t != '\0')) {
 		if ((*s == *t) || (*s == '*') || (*t == '^')) {
 			s++;
 			t++;
-		} else return FALSE;
+		} else return false;
 	}
 	while ((*s != '\0') && (*s == '*')) s++;
 	return (*s == '\0');
@@ -599,26 +599,28 @@ static int hash_disjunct(disjunct_dup_table *dt, Disjunct * d)
  * if this happens, it constitutes a proof that there is absolutely
  * no use for d2.
  */
-static int disjunct_matches_alam(Disjunct * d1, Disjunct * d2)
+static bool disjunct_matches_alam(Disjunct * d1, Disjunct * d2)
 {
 	Connector *e1, *e2;
-	if (d1->cost > d2->cost) return FALSE;
+	if (d1->cost > d2->cost) return false;
 	e1 = d1->left;
 	e2 = d2->left;
-	while((e1!=NULL) && (e2!=NULL)) {
+	while ((e1!=NULL) && (e2!=NULL))
+	{
 		if (!connector_matches_alam(e1,e2)) break;
 		e1 = e1->next;
 		e2 = e2->next;
 	}
-	if ((e1!=NULL) || (e2!=NULL)) return FALSE;
+	if ((e1!=NULL) || (e2!=NULL)) return false;
 	e1 = d1->right;
 	e2 = d2->right;
-	while((e1!=NULL) && (e2!=NULL)) {
+	while ((e1!=NULL) && (e2!=NULL))
+	{
 		if (!connector_matches_alam(e1,e2)) break;
 		e1 = e1->next;
 		e2 = e2->next;
 	}
-	if ((e1!=NULL) || (e2!=NULL)) return FALSE;
+	if ((e1!=NULL) || (e2!=NULL)) return false;
 	return (strcmp(d1->string, d2->string) == 0);
 }
 
@@ -816,17 +818,17 @@ static inline bool matches_S(connector_table *ct, Connector * c, char dir)
 	{
 		for (e = ct[h]; e != NULL; e = e->tableNext)
 		{
-			if (prune_match(0, e, c)) return TRUE;
+			if (prune_match(0, e, c)) return true;
 		}
-		return FALSE;
+		return false;
 	}
 	else
 	{
 		for (e = ct[h]; e != NULL; e = e->tableNext)
 		{
-			if (prune_match(0, c, e)) return TRUE;
+			if (prune_match(0, c, e)) return true;
 		}
-		return FALSE;
+		return false;
 	}
 }
 
@@ -1132,7 +1134,7 @@ static void power_table_delete(power_table *pt)
  * The disjunct d (whose left or right pointer points to c) is put
  * into the appropriate hash table
  */
-static void put_into_power_table(unsigned int size, C_list ** t, Connector * c, Boolean shal)
+static void put_into_power_table(unsigned int size, C_list ** t, Connector * c, bool shal)
 {
 	unsigned int h;
 	C_list * m;
@@ -1219,9 +1221,9 @@ static power_table * power_table_new(Sentence sent)
 		for (d=sent->word[w].d; d!=NULL; d=d->next) {
 			c = d->left;
 			if (c != NULL) {
-				put_into_power_table(size, t, c, TRUE);
+				put_into_power_table(size, t, c, true);
 				for (c=c->next; c!=NULL; c=c->next){
-					put_into_power_table(size, t, c, FALSE);
+					put_into_power_table(size, t, c, false);
 				}
 			}
 		}
@@ -1236,9 +1238,9 @@ static power_table * power_table_new(Sentence sent)
 		for (d=sent->word[w].d; d!=NULL; d=d->next) {
 			c = d->right;
 			if (c != NULL) {
-				put_into_power_table(size, t, c, TRUE);
+				put_into_power_table(size, t, c, true);
 				for (c=c->next; c!=NULL; c=c->next){
-					put_into_power_table(size, t, c, FALSE);
+					put_into_power_table(size, t, c, false);
 				}
 			}
 		}
@@ -1277,14 +1279,14 @@ static void clean_table(unsigned int size, C_list ** t)
  * possible for these two to match based on local considerations.
  */
 #ifdef USE_FAT_LINKAGES
-static Boolean possible_connection(prune_context *pc,
+static bool possible_connection(prune_context *pc,
                                Connector *lc, Connector *rc,
-                               Boolean lshallow, Boolean rshallow,
+                               bool lshallow, bool rshallow,
                                int lword, int rword)
 {
-	if ((!lshallow) && (!rshallow)) return FALSE;
+	if ((!lshallow) && (!rshallow)) return false;
 	  /* two deep connectors can't work */
-	if ((lc->word > rword) || (rc->word < lword)) return FALSE;
+	if ((lc->word > rword) || (rc->word < lword)) return false;
 	  /* word range constraints */
 
 	assert(lword < rword, "Bad word order in possible connection.");
@@ -1297,24 +1299,24 @@ static Boolean possible_connection(prune_context *pc,
 
 	if (pc->power_prune_mode == RUTHLESS) {
 		if (lword == rword-1) {
-			if (!((lc->next == NULL) && (rc->next == NULL))) return FALSE;
+			if (!((lc->next == NULL) && (rc->next == NULL))) return false;
 		} else {
 			if ((!pc->null_links) &&
 				(lc->next == NULL) && (rc->next == NULL) && (!lc->multi) && (!rc->multi)) {
-				return FALSE;
+				return false;
 			}
 		}
 		return do_match(pc->sent->count_ctxt, lc, rc, lword, rword);
 	} else {
 		if (lword == rword-1) {
-			if (!((lc->next == NULL) && (rc->next == NULL))) return FALSE;
+			if (!((lc->next == NULL) && (rc->next == NULL))) return false;
 		} else {
 			if ((!pc->null_links) &&
 				(lc->next == NULL) && (rc->next == NULL) && (!lc->multi) && (!rc->multi)
 				&& !pc->deletable[lword][rword]
 				)
 			{
-				return FALSE;
+				return false;
 			}
 		}
 		return prune_match(pc->effective_dist[lword][rword], lc, rc);
@@ -1324,7 +1326,7 @@ static Boolean possible_connection(prune_context *pc,
 
 static bool possible_connection(prune_context *pc,
                                Connector *lc, Connector *rc,
-                               Boolean lshallow, Boolean rshallow,
+                               bool lshallow, bool rshallow,
                                int lword, int rword)
 {
 	int dist;
@@ -1359,9 +1361,9 @@ static bool possible_connection(prune_context *pc,
  * This returns TRUE if the right table of word w contains
  * a connector that can match to c.  shallow tells if c is shallow.
  */
-static Boolean
+static bool
 right_table_search(prune_context *pc, int w, Connector *c,
-                   Boolean shallow, int word_c)
+                   bool shallow, int word_c)
 {
 	unsigned int size, h;
 	C_list *cl;
@@ -1374,19 +1376,19 @@ right_table_search(prune_context *pc, int w, Connector *c,
 	{
 		if (possible_connection(pc, cl->c, c, cl->shallow, shallow, w, word_c))
 		{
-			return TRUE;
+			return true;
 		}
 	}
-	return FALSE;
+	return false;
 }
 
 /**
  * This returns TRUE if the right table of word w contains
  * a connector that can match to c.  shallows tells if c is shallow
  */
-static Boolean
+static bool
 left_table_search(prune_context *pc, int w, Connector *c,
-                  Boolean shallow, int word_c)
+                  bool shallow, int word_c)
 {
 	unsigned int size, h;
 	C_list *cl;
@@ -1398,9 +1400,9 @@ left_table_search(prune_context *pc, int w, Connector *c,
 	for (cl = pt->l_table[w][h]; cl != NULL; cl = cl->next)
 	{
 		if (possible_connection(pc, c, cl->c, shallow, cl->shallow, word_c, w))
-			return TRUE;
+			return true;
 	}
-	return FALSE;
+	return false;
 }
 
 /**
@@ -1413,25 +1415,28 @@ left_table_search(prune_context *pc, int w, Connector *c,
  */
 static int
 left_connector_list_update(prune_context *pc, Connector *c,
-                           int word_c, int w, Boolean shallow)
+                           int word_c, int w, bool shallow)
 {
 	int n;
-	int foundmatch;
+	bool foundmatch;
 
 	if (c == NULL) return w;
-	n = left_connector_list_update(pc, c->next, word_c, w, FALSE) - 1;
+	n = left_connector_list_update(pc, c->next, word_c, w, false) - 1;
 	if (((int) c->word) < n) n = c->word;
 
 	/* n is now the rightmost word we need to check */
-	foundmatch = FALSE;
-	for (; n >= 0 ; n--) {
+	foundmatch = false;
+	for (; n >= 0 ; n--)
+	{
 		pc->power_cost++;
-		if (right_table_search(pc, n, c, shallow, word_c)) {
-			foundmatch = TRUE;
+		if (right_table_search(pc, n, c, shallow, word_c))
+		{
+			foundmatch = true;
 			break;
 		}
 	}
-	if (n < ((int) c->word)) {
+	if (n < ((int) c->word))
+	{
 		c->word = n;
 		pc->N_changed++;
 	}
@@ -1454,15 +1459,17 @@ right_connector_list_update(prune_context *pc, Sentence sent, Connector *c,
 	bool foundmatch;
 
 	if (c == NULL) return w;
-	n = right_connector_list_update(pc, sent, c->next, word_c, w, FALSE) + 1;
+	n = right_connector_list_update(pc, sent, c->next, word_c, w, false) + 1;
 	if (c->word > n) n = c->word;
 
 	/* n is now the leftmost word we need to check */
-	foundmatch = FALSE;
-	for (; n < sent->length ; n++) {
+	foundmatch = false;
+	for (; n < sent->length ; n++)
+	{
 		pc->power_cost++;
-		if (left_table_search(pc, n, c, shallow, word_c)) {
-			foundmatch = TRUE;
+		if (left_table_search(pc, n, c, shallow, word_c))
+		{
+			foundmatch = true;
 			break;
 		}
 	}
@@ -1474,7 +1481,11 @@ right_connector_list_update(prune_context *pc, Sentence sent, Connector *c,
 }
 
 /** The return value is the number of disjuncts deleted */
+#ifdef USE_FAT_LINKAGES
 int power_prune(Sentence sent, int mode, Parse_Options opts)
+#else
+int power_prune(Sentence sent, Parse_Options opts)
+#endif /* USE_FAT_LINKAGES */
 {
 	power_table *pt;
 	prune_context *pc;
@@ -1510,7 +1521,7 @@ int power_prune(Sentence sent, int mode, Parse_Options opts)
 			if ((2 == w%7) && parse_options_resources_exhausted(opts)) break;
 			for (d = sent->word[w].d; d != NULL; d = d->next) {
 				if (d->left == NULL) continue;
-				if (left_connector_list_update(pc, d->left, w, w, TRUE) < 0) {
+				if (left_connector_list_update(pc, d->left, w, w, true) < 0) {
 					for (c=d->left;  c != NULL; c = c->next) c->word = BAD_WORD;
 					for (c=d->right; c != NULL; c = c->next) c->word = BAD_WORD;
 					N_deleted++;
@@ -1545,7 +1556,7 @@ int power_prune(Sentence sent, int mode, Parse_Options opts)
 			if ((2 == w%7) && parse_options_resources_exhausted(opts)) break;
 			for (d = sent->word[w].d; d != NULL; d = d->next) {
 				if (d->right == NULL) continue;
-				if (right_connector_list_update(pc, sent, d->right, w, w, TRUE) >= sent->length) {
+				if (right_connector_list_update(pc, sent, d->right, w, w, true) >= sent->length) {
 					for (c=d->right; c != NULL; c = c->next) c->word = BAD_WORD;
 					for (c=d->left;  c != NULL; c = c->next) c->word = BAD_WORD;
 					N_deleted++;
@@ -1578,9 +1589,10 @@ int power_prune(Sentence sent, int mode, Parse_Options opts)
 	power_table_delete(pt);
 	pt = NULL;
 	pc->pt = NULL;
-	
+
 	if (verbosity > 2) printf("%d power prune cost:\n", pc->power_cost);
 
+#ifdef USE_FAT_LINKAGES
 	if (mode == RUTHLESS) {
 		print_time(opts, "power pruned (ruthless)");
 	} else {
@@ -1595,6 +1607,13 @@ int power_prune(Sentence sent, int mode, Parse_Options opts)
 		}
 		print_disjunct_counts(sent);
 	}
+#else
+	print_time(opts, "power pruned");
+	if (verbosity > 2) {
+		printf("\nAfter power_pruning:\n");
+		print_disjunct_counts(sent);
+	}
+#endif /* USE_FAT_LINKAGES */
 
 	xfree(pc, sizeof(prune_context));
 	return total_deleted;
@@ -1724,14 +1743,14 @@ static unsigned int cms_hash(const char * s)
  * This returns TRUE if there is a connector name C in the table
  * such that post_process_match(pp_match_name, C) is TRUE
  */
-static int match_in_cms_table(multiset_table *cmt, const char * pp_match_name)
+static bool match_in_cms_table(multiset_table *cmt, const char * pp_match_name)
 {
 	Cms * cms;
 	for (cms = cmt->cms_table[cms_hash(pp_match_name)]; cms != NULL; cms = cms->next)
 	{
-		if(post_process_match(pp_match_name, cms->name)) return TRUE;
+		if (post_process_match(pp_match_name, cms->name)) return true;
 	}
-	return FALSE;
+	return false;
 }
 
 static Cms * lookup_in_cms_table(multiset_table *cmt, const char * str)
@@ -1766,18 +1785,18 @@ static void insert_in_cms_table(multiset_table *cmt, const char * str)
  * Delete the given string from the table.  Return TRUE if
  * this caused a count to go to 0, return FALSE otherwise.
  */
-static int delete_from_cms_table(multiset_table *cmt, const char * str)
+static bool delete_from_cms_table(multiset_table *cmt, const char * str)
 {
-	Cms * cms;
-	cms = lookup_in_cms_table(cmt, str);
-	if (cms != NULL && cms->count > 0) {
+	Cms * cms = lookup_in_cms_table(cmt, str);
+	if (cms != NULL && cms->count > 0)
+	{
 		cms->count--;
 		return (cms->count == 0);
 	}
-	return FALSE;
+	return false;
 }
 
-static Boolean rule_satisfiable(multiset_table *cmt, pp_linkset *ls)
+static bool rule_satisfiable(multiset_table *cmt, pp_linkset *ls)
 {
 	unsigned int hashval;
 	const char * t;
@@ -1828,23 +1847,18 @@ static Boolean rule_satisfiable(multiset_table *cmt, pp_linkset *ls)
 			/* now if bad==0 this criterion link does the job
 			   to satisfy the needs of the trigger link */
 
-			if (bad == 0) return TRUE;
+			if (bad == 0) return true;
 		}
 	}
-	return FALSE;
+	return false;
 }
 
 static int pp_prune(Sentence sent, Parse_Options opts)
 {
 	pp_knowledge * knowledge;
-	pp_rule rule;
-	const char * selector;
-	pp_linkset * link_set;
 	size_t i, w;
-	char dir;
-	Disjunct *d;
-	Connector *c;
-	int change, total_deleted, N_deleted, deleteme;
+	int total_deleted, N_deleted;
+	bool change, deleteme;
 	multiset_table *cmt;
 
 	if (sent->dict->postprocessor == NULL) return 0;
@@ -1853,11 +1867,18 @@ static int pp_prune(Sentence sent, Parse_Options opts)
 
 	cmt = cms_table_new();
 
-	for (w = 0; w < sent->length; w++) {
-		for (d = sent->word[w].d; d != NULL; d = d->next) {
-			d->marked = TRUE;
-			for (dir=0; dir < 2; dir++) {
-				for (c = ( (dir)?(d->left):(d->right) ); c!=NULL; c=c->next) {
+	for (w = 0; w < sent->length; w++)
+	{
+		Disjunct *d;
+		for (d = sent->word[w].d; d != NULL; d = d->next)
+		{
+			char dir;
+			d->marked = true;
+			for (dir=0; dir < 2; dir++)
+			{
+				Connector *c;
+				for (c = ((dir) ? (d->left) : (d->right)); c != NULL; c = c->next)
+				{
 					insert_in_cms_table(cmt, c->string);
 				}
 			}
@@ -1865,21 +1886,29 @@ static int pp_prune(Sentence sent, Parse_Options opts)
 	}
 
 	total_deleted = 0;
-	change = 1;
-	while (change > 0) {
-		change = 0;
+	change = true;
+	while (change)
+	{
+		change = false;
 		N_deleted = 0;
-		for (w = 0; w < sent->length; w++) {
-			for (d = sent->word[w].d; d != NULL; d = d->next) {
+		for (w = 0; w < sent->length; w++)
+		{
+			Disjunct *d;
+			for (d = sent->word[w].d; d != NULL; d = d->next)
+			{
+				char dir;
 				if (!d->marked) continue;
-				deleteme = FALSE;
-				for (dir=0; dir < 2; dir++) {
-					for (c = ( (dir)?(d->left):(d->right) ); c!=NULL; c=c->next) {
-						for (i=0; i<knowledge->n_contains_one_rules; i++) {
-
-							rule = knowledge->contains_one_rules[i]; /* the ith rule */
-							selector = rule.selector;				/* selector string for this rule */
-							link_set = rule.link_set;				/* the set of criterion links */
+				deleteme = false;
+				for (dir = 0; dir < 2; dir++)
+				{
+					Connector *c;
+					for (c = ((dir) ? (d->left) : (d->right)); c != NULL; c = c->next)
+					{
+						for (i = 0; i < knowledge->n_contains_one_rules; i++)
+						{
+							pp_rule* rule = &knowledge->contains_one_rules[i]; /* the ith rule */
+							const char * selector = rule->selector;  /* selector string for this rule */
+							pp_linkset * link_set = rule->link_set;  /* the set of criterion links */
 
 							if (strchr(selector, '*') != NULL) continue;  /* If it has a * forget it */
 
@@ -1892,8 +1921,10 @@ static int pp_prune(Sentence sent, Parse_Options opts)
 							/* We know c matches the trigger link of the rule. */
 							/* Now check the criterion links */
 
-							if (!rule_satisfiable(cmt, link_set)) {
-								deleteme = TRUE;
+							if (!rule_satisfiable(cmt, link_set))
+							{
+								deleteme = true;
+								rule->use_count++;
 							}
 							if (deleteme) break;
 						}
@@ -1902,28 +1933,34 @@ static int pp_prune(Sentence sent, Parse_Options opts)
 					if (deleteme) break;
 				}
 
-				if (deleteme) {					/* now we delete this disjunct */
+				if (deleteme)         /* now we delete this disjunct */
+				{
+					char dir;
 					N_deleted++;
 					total_deleted++;
-					d->marked = FALSE; /* mark for deletion later */
-					for (dir=0; dir < 2; dir++) {
-						for (c = ( (dir)?(d->left):(d->right) ); c!=NULL; c=c->next) {
-							change += delete_from_cms_table(cmt, c->string);
+					d->marked = false; /* mark for deletion later */
+					for (dir=0; dir < 2; dir++)
+					{
+						Connector *c;
+						for (c = ((dir) ? (d->left) : (d->right)); c != NULL; c = c->next)
+						{
+							change |= delete_from_cms_table(cmt, c->string);
 						}
 					}
 				}
 			}
 		}
 
-		if (verbosity > 2) {
+		if (verbosity > 2)
+		{
 			printf("pp_prune pass deleted %d\n", N_deleted);
 		}
-
 	}
 	delete_unmarked_disjuncts(sent);
 	cms_table_delete(cmt);
 
-	if (verbosity > 2) {
+	if (verbosity > 2)
+	{
 		printf("\nAfter pp_pruning:\n");
 		print_disjunct_counts(sent);
 	}
@@ -1939,6 +1976,7 @@ static int pp_prune(Sentence sent, Parse_Options opts)
  * power pp power pp power pp....
  * Make sure you do them both at least once.
  */
+#ifdef USE_FAT_LINKAGES
 void pp_and_power_prune(Sentence sent, int mode, Parse_Options opts)
 {
 	power_prune(sent, mode, opts);
@@ -1950,4 +1988,16 @@ void pp_and_power_prune(Sentence sent, int mode, Parse_Options opts)
 		if (power_prune(sent, mode, opts) == 0) break;
 	}
 }
+#else
+void pp_and_power_prune(Sentence sent, Parse_Options opts)
+{
+	power_prune(sent, opts);
 
+	for (;;) {
+		if (parse_options_resources_exhausted(opts)) break;
+		if (pp_prune(sent, opts) == 0) break;
+		if (parse_options_resources_exhausted(opts)) break;
+		if (power_prune(sent, opts) == 0) break;
+	}
+}
+#endif /* USE_FAT_LINKAGES */
